@@ -33,14 +33,17 @@ class DynamicSerializerMethodField(SerializerMethodField):
         return method(value, query)
 
 
-class BaseReplaceableNestedField(object):
+class BaseRESTQLNestedField(object):
     def to_internal_value(self, data):
         raise NotImplementedError('`to_internal_value()` must be implemented.')
 
 
-class BaseWritableNestedField(object):
-    def to_internal_value(self, data):
-        raise NotImplementedError('`to_internal_value()` must be implemented.')
+class BaseReplaceableNestedField(BaseRESTQLNestedField):
+    pass
+
+
+class BaseWritableNestedField(BaseRESTQLNestedField):
+    pass
 
 
 def BaseNestedFieldSerializerFactory(
@@ -192,11 +195,6 @@ def BaseNestedFieldSerializerFactory(
             return data
 
         def to_internal_value(self, data):
-            source = kwargs.get('source', None)
-            if source is not None and self.parent is not None:
-                alias = self.field_name
-                self.parent._aliased_nested_fields.update({source: self})
-                self.parent._unaliased_fields.pop(alias)
             required = kwargs.get('required', True)
             request = self.context.get('request')
             if request.method in ["PUT", "PATCH"]:
@@ -253,14 +251,12 @@ def BaseNestedFieldSerializerFactory(
             return parent_serializer.validated_data
 
         def to_internal_value(self, data):
-            source = kwargs.get('source', None)
-            if source is not None and self.parent is not None:
-                alias = self.field_name
-                self.parent._aliased_nested_fields.update({source: self})
-                self.parent._unaliased_fields.pop(alias)
             required = kwargs.get('required', True)
             default = kwargs.get('default', empty)
 
+            if data == empty and self.is_partial:
+                return empty
+                
             if data == empty and required and default == empty:
                 raise ValidationError(
                     "This field is required.",
